/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - March 20, 2000.
   -
   - The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
   - Portions created by Henk-Johan van Rantwijk are Copyright (C) 2000
   - Henk-Johan van Rantwijk.  All Rights Reserved.
   -
   - Contributor(s):
   -  Henk-Johan van Rantwijk <bugs4hj@netscape.net> (Original Author)
   -
   - Alternatively, the contents of this file may be used under the
   - terms of the GNU General Public License Version 2 or later (the
   - "GPL"), in which case the provisions of the GPL are applicable
   - instead of those above.  If you wish to allow use of your
   - version of this file only under the terms of the GPL and not to
   - allow others to use your version of this file under the MPL,
   - indicate your decision by deleting the provisions above and
   - replace them with the notice and other provisions required by
   - the GPL.  If you do not delete the provisions above, a recipient
   - may use your version of this file under either the MPL or the
   - GPL.
   -
*/

var multiviewsBundle = "chrome://multiviews/locale/multiviews.properties";
var RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
var gdLocalStore = null;
var mzToolbox = null;

const gbToolboxVisibilityPrefListener =
{
  domain: "multizilla.integrated.toolbar",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var isVisible = pref.getBoolPref(prefName);
    mzToolbox.mzToolboxSetVisibility(isVisible);
  }
};
const gTabMinWidthPrefListener =
{
  domain: "multizilla.tabs.mintabwidth",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabWidth = pref.getIntPref(prefName);
    gBrowser.setTabWidthTo("minwidth", tabWidth);
  }
};
const gTabWidthPrefListener =
{
  domain: "multizilla.tabs.tabwidth",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabWidth = pref.getIntPref(prefName);
    gBrowser.setTabWidthTo("maxwidth", tabWidth);
  }
};
const gTabLEDstatePrefListener =
{
  domain: "multizilla.tabs.status-leds",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabLEDstate = pref.getBoolPref(prefName);
    gBrowser.setTabLEDstateTo(tabLEDstate);
  }
};
const gTabLabelPrefListener =
{
  domain: "multizilla.tabs.title",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabLabel = pref.getBoolPref(prefName);
    gBrowser.setTabLabelsTo(tabLabel);
  }
};
const gTabSwitchPrefListener =
{
  domain: "multizilla.tabs.ontop",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var topTabs = pref.getBoolPref(prefName);
    if (topTabs)
      gBrowser.activateTopTabs()
    else gBrowser.activateBottomTabs();
  }
};
const gTabCanClosePrefListener =
{
  domain: "multizilla.tabs.iconclose",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabCanClose = pref.getBoolPref(prefName);
    gBrowser.setTabCanClose(tabCanClose);
  }
};
const gTBButtonPrefListener =
{
  domain: "multizilla.showbutton",
  observe: function(subject, topic, prefName)   {
    if (topic != "nsPref:changed")
      return;
    var buttonName = prefName.substr(this.domain.length+1);
    var buttonId = buttonName + "-tbbutton";
    var button = document.getElementById(buttonId);
    var show = pref.getBoolPref(prefName);

    if (show) 
      button.removeAttribute("hidden");
    else button.setAttribute("hidden", "true");
  }
};
const gOrdinancePrefListener =
{
  domain: "multizilla.tabs.open-order",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var ordinance = pref.getIntPref(prefName);
    gBrowser.mTabOrdinance = ordinance;
  }
};
const gtabFocusOrderPrefListener =
{
  domain: "multizilla.tabs.focus-order",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var tabFocusOrder = pref.getIntPref(prefName);
    gBrowser.mTabFocus = tabFocusOrder;
  }
};
const gtabbarCloseButtonPrefListener =
{
  domain: "multizilla.tabbar.closebutton",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var closeButton = pref.getBoolPref(prefName);
    gBrowser.mTabContainer.setAttribute("displayclose", closeButton ? "true" : "false");
  }
};
const gtabbarNewButtonPrefListener =
{
  domain: "multizilla.tabbar.newbutton",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    var newButton = pref.getBoolPref(prefName);
    gBrowser.mTabContainer.setAttribute("displaynew", newButton ? "true" : "false");
  }
};
const gTabSiteIconsPrefListener =
{
  domain: "browser.chrome.site_icons",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    gBrowser.updateDisplayTabIcons();
  }
};
const gTabFaviconsPrefListener =
{
  domain: "browser.chrome.favicons",
  observe: function(subject, topic, prefName) {
    if (topic != "nsPref:changed")
      return;

    gBrowser.updateDisplayTabIcons();
  }
};
////////////////////////////////////////////////////////////////////////////////
function confirmCloseWS(aConfirm)
{
  var confirmPref = readMyPref("multizilla.confirm.quit", "int", 0);
  var closeBundle = srGetStrBundle("chrome://multiviews/locale/multiviews.properties");
  var aHeader = closeBundle.GetStringFromName("closeSession");
  var aQuestion = closeBundle.GetStringFromName("closeSessionQuestion");
  var tabs = gBrowser.mPanelContainer.childNodes.length;
  /* ::::
     1  = File Menu 'Close Window' & Ctrl-Shift-W
     2  = File Menu 'Exit' & Ctrl-Q
     4  = Click on [X] on Window
     8  = JavaScript close()
     16 = no confirmation for single tab
  :::: */
  if (tabs == 1 && confirmPref & 16) {
    checkGroupSaving();
    removeIdentifier();
    return true;
  }
  else if (aConfirm == 8 && document.documentElement.hasAttribute("chromehidden")) {
    removeIdentifier();
    return true;
  }
  else if (confirmPref && (confirmPref & aConfirm) == aConfirm) {
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                        getService(Components.interfaces.nsIPromptService);

    if (aConfirm & 1) {
      aHeader = closeBundle.GetStringFromName("closeWindow");
      aQuestion = closeBundle.GetStringFromName("closeWindowQuestion");
    }
    if (promptService.confirm(window, aHeader, aQuestion)) {
      checkGroupSaving();
      removeIdentifier();
      return true;
    }
    else
      return false;
  }
  checkGroupSaving();
  removeIdentifier(); 
  return true;
}
////////////////////////////////////////////////////////////////////////////////
function multiZillaInit()
{
  addPrefListener(gTabMinWidthPrefListener);
  addPrefListener(gTabWidthPrefListener);
  addPrefListener(gTabLEDstatePrefListener);
  addPrefListener(gTabLabelPrefListener);
  addPrefListener(gTabSwitchPrefListener);
  addPrefListener(gTBButtonPrefListener);
  addPrefListener(gTabCanClosePrefListener);
  addPrefListener(gbToolboxVisibilityPrefListener);
  addPrefListener(gOrdinancePrefListener);
  addPrefListener(gtabFocusOrderPrefListener);
  addPrefListener(gtabbarNewButtonPrefListener);
  addPrefListener(gtabbarCloseButtonPrefListener);
  addPrefListener(gTabSiteIconsPrefListener);
  addPrefListener(gTabFaviconsPrefListener);
  initPreferences();
}
////////////////////////////////////////////////////////////////////////////////
// this function sets ID's to the Go Menu and replaces the popup
// I know this is a dirty way to do it, but we need to ;)
////////////////////////////////////////////////////////////////////////////////
function addGoMenuIDs()
{
  var goMenu = document.getElementById("main-menubar").childNodes[3];
  goMenu.setAttribute("id", "menu_Go");
  var goMenuPopup = document.getElementById("menu_GoPopup");
  goMenu.firstChild.setAttribute("id", "menu_GoPopup");
  goMenu.replaceChild(goMenuPopup, goMenu.firstChild);

  // added for Mac(OSX)Zilla support
  initializeBookmarksTemplate();
  initializePersonalToolbarTemplate();

  // replace Bookmarks menu template
  var mainMenu = document.getElementById("main-menubar");
  var newBookmarkMenu = document.getElementById("NewBookmarksMenu");
  var clonedMenu = newBookmarkMenu.cloneNode(true);

  // main bookmarks menu modifications
  var menupopupList = newBookmarkMenu.getElementsByTagName("menupopup");

  for (var i = 0; i < menupopupList.length; i++) {
    if (menupopupList[i].hasAttribute("class"))
      menupopupList[i].setAttribute("class", "mainbookmarks-menupopup");
  }
  menupopupList = clonedMenu.getElementsByTagName("menupopup");
  menupopupList[i-1].setAttribute("class", "mainbookmarks-menupopup");
  mainMenu.replaceChild(clonedMenu, mainMenu.childNodes[4]);
  clonedMenu.setAttribute("id", "BookmarksMenu");
  clonedMenu.setAttribute("hidden", "false"); 

  // Set Mac platform identifier
  var platform = window.navigator.platform;
  var regExp = new RegExp("Mac", "i");
  var onMac = platform.search(regExp) ? false : true;

  if (!onMac) { // Modify Main Bookmarks menu
    clonedMenu.removeAttribute("oncommand");
    clonedMenu.setAttribute("onclick", "if(event.button==0){OpenBookmarkURL(event.target, this.database, event)}");
  }
  // remove the fake bookmarks menu item from the tabmenu
  var tabMenu = document.getElementById("tabContextMenu");
  tabMenu.removeChild(newBookmarkMenu);

  // added for Mac(OSX)Zilla support
  var bookmarkButton = document.getElementById("bookmarks-button");
  bookmarkButton.firstChild.setAttribute("class","bookmarks-menupopup");
  var originalPTBox = document.getElementById("NC:PersonalToolbarFolder");
  var newPTBox = document.getElementById("NEW-NC:PersonalToolbarFolder");
  var clonedTemplate = newPTBox.childNodes[1].cloneNode(true);
  originalPTBox.replaceChild(clonedTemplate, originalPTBox.childNodes[1]);
  tabMenu.removeChild(newPTBox);

  // added for the integrated toolbar
  var shouldDisplay = readMyPref("multizilla.integrated.toolbar", "bool", false);
  mzToolbox = document.getElementById("MultiZillaToolbar");
  mzToolbox.mzToolboxSetVisibility(shouldDisplay);

  // Initialize ordinals
  var nodeList = document.getAnonymousNodes(getBrowser().mTabContainer);
  var index = onMac ? 1 : 0;

  for (i = (0+index); i < nodeList.length; i++) {
    nodeList[i].ordinal = i;
    if (nodeList[i].localName == "hbox") {
      nodeList[i].firstChild.ordinal = 0;
      nodeList[i].lastChild.ordinal = 9999999;
    }
  }
  nodeList[nodeList.length-(1+index)].ordinal = 9999999;
}
////////////////////////////////////////////////////////////////////////////////
function isFirstWindow()
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var topWindow = windowManagerInterface.getMostRecentWindow(null);
  var enumerator = windowManagerInterface.getEnumerator("navigator:browser");
  var firstWindow = enumerator.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);

  if (firstWindow != topWindow)
    return (false);
  return (true);
}
////////////////////////////////////////////////////////////////////////////////
function loadTabGroup(aCheckFirstWindow)
{
  if (readMyPref("multizilla.prevent.double", "bool", true)) {
    var delayed = false;
    var tab = null;
    if (pref.getIntPref("browser.startup.page") == 1) {
      var pageCount = pref.getIntPref("browser.startup.homepage.count");
      tab = gBrowser.mTabContainer.childNodes;
      if (pageCount != tab.length)
        delayed = true;
      for (var i = 0; i < tab.length; i++) {
        if (tab[i].hasAttribute("busy")) {
          delayed = true;
          break;
        }
      }
    }
    if (delayed && aCheckFirstWindow) {
      setTimeout(loadTabGroup, 1000, true);
      return(index);
    }
  }
  gdLocalStore = RDF.GetDataSource("rdf:local-store");
  var index = 0;
  if ((!isFirstWindow() && aCheckFirstWindow) || !gdLocalStore)
    return(index);
  var RDFC = Components.classes["@mozilla.org/rdf/container-utils;1"].getService(Components.interfaces.nsIRDFContainerUtils);
  var urls = RDFC.MakeSeq(gdLocalStore, RDF.GetResource("nc:group-url")).GetElements();
  var labels = RDFC.MakeSeq(gdLocalStore, RDF.GetResource("nc:group-label")).GetElements();
  if (!urls.hasMoreElements())
    return(index);

  while (urls.hasMoreElements()) {
    var url = urls.getNext();
    var label = labels.getNext();
    if (url && label) {
      try {
        url = url.QueryInterface(Components.interfaces.nsIRDFLiteral);
        label = label.QueryInterface(Components.interfaces.nsIRDFLiteral);
        index++;
      } catch(ex) {
        continue;
      }
      if (!preventDupeTab(url.Value)) {
        if (index == 1 && useFirstPage())
          loadURI(url.Value, null);
        else
          getBrowser().addTab(url.Value, null);
        tab = getBrowser().mTabContainer.lastChild;
        if (label.Value != "false") {
          tab.setAttribute("customLabel", "true");
          tab.setAttribute("label", label.Value);
        }
      }
    }
  }
  return(index);
}
////////////////////////////////////////////////////////////////////////////////
function checkGroupSaving()
{
  var shouldSave = readMyPref("multizilla.tabs.saveonexit", "bool", false);
  var tabCount = getBrowser().mPanelContainer.childNodes.length;

  if (!shouldSave)
    return;
  else storeGroupData();
}
////////////////////////////////////////////////////////////////////////////////
function useFirstPage()
{
 var uriToLoad = null;
  if ("arguments" in window && window.arguments.length >= 1 && window.arguments[0])
    uriToLoad = window.arguments[0];
  if (readMyPref("multizilla.tabs.andnotblank", "bool", true)) {
    if ((readMyPref("browser.startup.page", "int", 1) == 1 && 
        getHomePage() == "about:blank" || getHomePage() == "") || uriToLoad == "about:blank" || 
        readMyPref("browser.startup.page", "int", 0) == 3 && uriToLoad == "about:blank")
      return true;
  }
  return false;
}